@@ -49,14 +49,13 @@ import pickle
 import sys
 import warnings
 
-import pydot
-
 from dd import _abc
 from dd import _parser
 from dd import _compat
 from dd._compat import items
 # inline:
 # import networkx
+# import pydot
 
 
 logger = logging.getLogger(__name__)
@@ -129,7 +128,7 @@ class BDD(_abc.BDD):
 
     Attributes:
       - `vars`: `dict` mapping `variables` to `int` levels
-      - `roots`: (optional) edges used by `to_nx`.
+      - `roots`: (optional) edges
       - `max_nodes`: raise `Exception` if this limit is reached.
         The default value is `sys.maxsize` in Python 3 and
         `sys.maxint` in Python 2. Increase it if needed.
@@ -145,23 +144,56 @@ class BDD(_abc.BDD):
     or call `update_predecessors` prior to calling `ite`.
     """
     # omitted docstrings are inheritted from `super()`
+    # "nat" below means "natural number",
+    # i.e., int >= 0
 
     def __init__(self, levels=None):
         if levels is None:
             levels = dict()
         _assert_valid_ordering(levels)
-        self._pred = dict()  # (i, low, high) -> u
-        self._succ = dict()  # u -> (i, low, high)
-        self._ref = dict()  # reference counters
-        self._min_free = 2  # all smaller positive integers used
-        self._ite_table = dict()  # (cond, high, low)
-        self.vars = dict()  # var_to_level
+        # (level, low, high) -> node
+        # tuple(nat, int, nat) -> nat
+        self._pred = dict()
+        # node -> (level, low, high)
+        # nat -> tuple(nat, int, nat)
+        self._succ = dict()
+        # reference counters
+        # node -> reference count
+        # nat -> nat
+        self._ref = dict()
+        # all smaller positive integers
+        # are used as node indices, and
+        # no larger integers are used
+        # as node indices
+        # nat
+        self._min_free = 2
+        # cache for ternary conditional
+        # ("ite" is an initialism for
+        #  "if-then-else")
+        # (condition, then, else) -> edge
+        # (int, int, int) -> int
+        self._ite_table = dict()
+        # mapping from variable names to
+        # levels of variables
+        # var name -> level
+        # str -> nat
+        self.vars = dict()
+        # mapping from levels of variables
+        # to variable names,
+        # inverse of `self.vars`
+        # level -> var name
+        # nat -> str
         self._level_to_var = dict()
-        self._init_terminal(len(self.vars))  # handle no vars
-        self._reordering_context = False  # for decorator nesting
-        self._last_len = None  # after last reordering
-        for var, level in items(levels):
+        # handle no vars
+        self._init_terminal(len(self.vars))
+        # for decorator nesting
+        self._reordering_context = False
+        # after last reordering
+        self._last_len = None
+        for var, level in levels.items():
             self.add_var(var, level)
+        # set of edges
+        # optional
         self.roots = set()
         try:
             # Python 2, for xrange
@@ -183,7 +215,11 @@ class BDD(_abc.BDD):
         """Assert that all remaining nodes are garbage."""
         self.decref(1)  # free ref from `self._init_terminal`
         self.collect_garbage()
-        assert all(v == 0 for v in self._ref.values()), self._ref
+        if not all(v == 0 for v in self._ref.values()):
+            raise AssertionError((
+                'There are nodes still referenced '
+                'upon shutdown. Details:\n'
+                '{_ref}').format(_ref=self._ref))
 
     def __len__(self):
         return len(self._succ)
@@ -226,16 +262,16 @@ class BDD(_abc.BDD):
     @property
     def ordering(self):
         raise DeprecationWarning(
-            'use `dd.BDD.vars` instead of `.ordering`')
+            'use `dd.bdd.BDD.vars` instead of `.ordering`')
 
-    def _init_terminal(self, i):
+    def _init_terminal(self, level):
         """Place constant node `1`.
 
         Used for initialization and to shift node `1` to
         lower levels, as fresh variables are being added.
         """
         u = 1
-        t = (i, None, None)
+        t = (level, None, None)
         told = self._succ.setdefault(u, t)
         self._pred.pop(told, None)
         self._succ[u] = t
@@ -252,26 +288,49 @@ class BDD(_abc.BDD):
 
     def decref(self, u):
         """Decrement reference count of node `u`, with 0 as min."""
-        if self._ref[abs(u)] > 0:
-            self._ref[abs(u)] -= 1
+        if self._ref[abs(u)] <= 0:
+            warnings.warn((
+                'The method `dd.bdd.BDD.decref` was called '
+                'for BDD node {u} with reference count {n}. '
+                'This call has no effect. Calling `decref` '
+                'for a node with nonpositive reference count '
+                'may indicate a programming error.'
+                ).format(u=u, n=self._ref[abs(u)]),
+                UserWarning)
+            return
+        self._ref[abs(u)] -= 1
 
     def ref(self, u):
         """Return reference count of edge `u`."""
         return self._ref[abs(u)]
 
     def add_var(self, var, level=None):
-        """Add a variable named `var` at `level`.
+        """Declare a variable named `var` at `level`.
 
-        If `level` is absent, then add at bottom.
-        Raise `Exception` if:
-            - `var` exists at different level, or
-            - `level` is occupied.
+        The new variable is Boolean-valued.
 
-        Currently, `add_var` must be called
-        *only* before adding any nodes.
-        In the future, this will change.
+        If `level` is absent, then add the new variable
+        at the bottom level.
 
+        Raise `ValueError` if:
+        - `var` already exists at a level
+          different than the given `level`, or
+        - the given `level` is already used by
+          another variable
+        - `level` is not given and `var` does not exist,
+          and the next level larger than the
+          current bottom level is already used by
+          another variable.
+
+        If `var` already exists, and either `level`
+        is not given, or `var` has `level`,
+        then return without raising exceptions.
+
+        @param var: name of new variable to declare
+        @type var: `str`
+        @param level: level of new variable to declare
         @type level: `int`
+        @return: level of variable `var`
         """
         # var already exists ?
         if var in self.vars:
@@ -356,7 +415,7 @@ class BDD(_abc.BDD):
     def descendants(self, roots):
         """Return nodes reachable from `roots`.
 
-        Nodes in `roots` are included.
+        Nodes pointed to by references in `roots` are included.
         Nodes are represented as positive integers.
         """
         if not roots:
@@ -787,9 +846,11 @@ class BDD(_abc.BDD):
         return w
 
     def find_or_add(self, i, v, w):
-        """Return a node at level `i` with successors `v, w`.
+        """Return reference to node at level `i` with successors `v, w`.
 
-        If one exists, it is quickly found in the cached table.
+        If such a node exists already,
+        then it is quickly found in the cached table,
+        and the reference returned.
 
         @param i: level in `range(n_vars - 1)`
         @param v: low edge
@@ -844,7 +905,6 @@ class BDD(_abc.BDD):
 
         @type roots: `set`, Caution: it is modified
         """
-        logger.debug('++ collecting BDD garbage')
         n = len(self)
         if roots is None:
             roots = self._ref
@@ -875,9 +935,6 @@ class BDD(_abc.BDD):
         m = len(self)
         k = n - m
         assert k >= 0, (n, m)
-        logger.debug(
-            '-- done: colected {n} - {m} = {k} nodes.'.format(
-                n=n, m=m, k=k))
 
     def update_predecessors(self):
         """Update table that maps (level, low, high) to nodes."""
@@ -1024,10 +1081,21 @@ class BDD(_abc.BDD):
         return (oldsize, newsize)
 
     def _low_high(self, u):
-        """Return low and high, or `u` itself, if terminal."""
+        """Return level, low, and high.
+
+        If node `u` is a leaf,
+        then `u` is returned as low and high.
+
+        This method is similar to the
+        method `succ`, but different.
+
+        @type u: `int`
+        @return: (level, low, high)
+        @rtype: `tuple(int, int, int)`
+        """
         i, v, w = self._succ[abs(u)]
         if abs(u) == 1:
-            return (i, u, u)
+            return i, u, u
         return i, v, w
 
     def _swap_cofactor(self, u, y):
@@ -1046,7 +1114,8 @@ class BDD(_abc.BDD):
 
     def count(self, u, nvars=None):
         n = nvars
-        assert abs(u) in self, u
+        if abs(u) not in self:
+            raise ValueError(u)
         # index those levels in support separately
         levels = {
             self.level_of_var(var)
@@ -1055,7 +1124,8 @@ class BDD(_abc.BDD):
         if n is None:
             n = k
         slack = n - k
-        assert slack >= 0, slack
+        if slack < 0:
+            raise ValueError(slack)
         map_level = dict()
         for new, old in enumerate(sorted(levels)):
             map_level[old] = new + slack
@@ -1218,26 +1288,46 @@ class BDD(_abc.BDD):
         assert v is None or abs(v) in self, v
         assert w is None or abs(w) in self, w
         if op in ('~', 'not', '!'):
+            assert v is None, v
+            assert w is None, w
             return -u
         elif op in ('or', r'\/', '|', '||'):
+            assert v is not None, v
+            assert w is None, w
             return self.ite(u, 1, v)
         elif op in ('and', '/\\', '&', '&&'):
+            assert v is not None, v
+            assert w is None, w
             return self.ite(u, v, -1)
         elif op in ('xor', '^'):
+            assert v is not None, v
+            assert w is None, w
             return self.ite(u, -v, v)
         elif op in ('=>', '->', 'implies'):
+            assert v is not None, v
+            assert w is None, w
             return self.ite(u, v, 1)
         elif op in ('<=>', '<->', 'equiv'):
+            assert v is not None, v
+            assert w is None, w
             return self.ite(u, v, -v)
         elif op in ('diff', '-'):
+            assert v is not None, v
+            assert w is None, w
             return self.ite(u, -v, -1)
-        elif op in ('\A', 'forall'):
+        elif op in (r'\A', 'forall'):
+            assert v is not None, v
+            assert w is None, w
             qvars = self.support(u)
             return self.quantify(v, qvars, forall=True)
-        elif op in ('\E', 'exists'):
+        elif op in (r'\E', 'exists'):
+            assert v is not None, v
+            assert w is None, w
             qvars = self.support(u)
             return self.quantify(v, qvars, forall=False)
         elif op == 'ite':
+            assert v is not None, v
+            assert w is not None, w
             return self.ite(u, v, w)
         else:
             raise Exception(
@@ -1272,9 +1362,11 @@ class BDD(_abc.BDD):
             elif name.endswith('.p'):
                 filetype = 'pickle'
             else:
-                raise Exception(
+                raise Exception((
                     'cannot infer file type '
-                    'from extension')
+                    'from extension of file '
+                    'name "{f}"').format(
+                        f=filename))
         if filetype in ('pdf', 'png', 'svg'):
             self._dump_figure(roots, filename,
                               filetype, **kw)
@@ -1788,7 +1880,8 @@ def copy_bdd(u, from_bdd, to_bdd):
     @type from_bdd, to_bdd: `BDD`
     """
     if from_bdd is to_bdd:
-        logger.warning('copying node to same manager')
+        logger.warning(
+            'copying node to same BDD manager')
         return u
     level_map = {
         from_bdd.level_of_var(var): to_bdd.level_of_var(var)
@@ -1843,7 +1936,7 @@ def _flip(r, u):
 
 
 def to_nx(bdd, roots):
-    """Convert functions in `roots` to `networkx.MultiDiGraph`.
+    """Convert node references in `roots` to `networkx.MultiDiGraph`.
 
     The resulting graph has:
 
@@ -1855,6 +1948,7 @@ def to_nx(bdd, roots):
 
     @type bdd: `BDD`
     @type roots: iterable of edges, each a signed `int`
+    @rtype: `networkx.MultiDiGraph`
     """
     import networkx as nx
     g = nx.MultiDiGraph()
@@ -1903,6 +1997,7 @@ def to_pydot(roots, bdd):
     @type roots: container of BDD nodes
     @type bdd: `BDD`
     """
+    import pydot
     # all nodes ?
     if roots is None:
         nodes = bdd._succ
