@@ -1,3 +1,4 @@
+"""Tests of the module `dd.bdd`."""
 import logging
 import os
 
@@ -5,9 +6,9 @@ from dd.bdd import BDD as _BDD
 from dd.bdd import preimage
 from dd import autoref
 from dd import bdd as _bdd
-import nose.tools as nt
 import networkx as nx
 import networkx.algorithms.isomorphism as iso
+import pytest
 
 
 class BDD(_BDD):
@@ -59,9 +60,9 @@ def test_add_var():
     assert b.vars['y'] == 5, b.vars
     assert j == 5, j
     # attempt to add var at an existing level
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         b.add_var('z', level=35)
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         b.add_var('z', level=5)
     #
     # mixing automated and
@@ -74,14 +75,14 @@ def test_add_var():
     assert 'y' in b.vars, b.vars
     assert b.vars['x'] == 2, b.vars
     assert b.vars['y'] == 1, b.vars
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         b.add_var('z')
     b.add_var('z', level=0)
 
 
 def test_var():
     b = BDD()
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         b.var('x')
     j = b.add_var('x')
     u = b.var('x')
@@ -98,17 +99,17 @@ def test_assert_consistent():
     g = x_or_y()
     assert g.assert_consistent()
     g._succ[2] = (5, 1, 2)
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.assert_consistent()
     g = x_or_y()
     g.roots.add(2)
     g._succ[4] = (0, 10, 1)
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.assert_consistent()
     g = x_or_y()
     g.roots.add(2)
     g._succ[1] = (2, None, 1)
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.assert_consistent()
     g = x_and_y()
     assert g.assert_consistent()
@@ -119,15 +120,24 @@ def test_level_to_variable():
     g = BDD(ordering)
     assert g.var_at_level(0) == 'x'
     assert g.var_at_level(1) == 'y'
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.var_at_level(10)
 
 
+def test_var_levels_attr():
+    bdd = BDD()
+    bdd.declare('x', 'y')
+    var_levels = bdd.var_levels
+    assert len(var_levels) == 2, var_levels
+    assert {'x', 'y'} == set(var_levels), var_levels
+    assert {0, 1} == set(var_levels.values()), var_levels
+
+
 def test_descendants():
     ordering = dict(x=0, y=1)
     b = BDD(ordering)
-    u = b.add_expr('x /\ y')
-    v = b.add_expr('x \/ y')
+    u = b.add_expr(r'x /\ y')
+    v = b.add_expr(r'x \/ y')
     roots = [u, v]
     nodes = b.descendants(roots)
     nodes_u = b.descendants([u])
@@ -199,14 +209,14 @@ def test_count():
     assert r == 6, r
     r = g.count(-4, 3)
     assert r == 2, r
-    with nt.assert_raises(Exception):
+    with pytest.raises(Exception):
         g.count()
     r = g.count(4)
     assert r == 3, r
     g = _bdd.BDD()
     g.add_var('x')
     g.add_var('y')
-    u = g.add_expr('x /\ y ')
+    u = g.add_expr(r'x /\ y ')
     r = g.count(u)
     assert r == 1, r
 
@@ -252,7 +262,7 @@ def test_pick_iter():
     # gaps in order
     order = {'x': 0, 'y': 1, 'z': 2}
     bdd = BDD(order)
-    u = bdd.add_expr('x /\ z')
+    u = bdd.add_expr(r'x /\ z')
     (m,) = bdd.pick_iter(u)
     assert m == {'x': 1, 'z': 1}, m
 
@@ -359,8 +369,6 @@ def test_reduce_combined():
 
     (u, ) = h.roots
     compare(u, h, r)
-    # r.write('r.pdf')
-    # h.write('h.pdf')
 
 
 def test_reduction_complemented_edges():
@@ -371,11 +379,11 @@ def test_reduction_complemented_edges():
     assert a < b, (a, b)
     # complemented edge from internal node to
     # non-terminal node
-    expr = '~ x /\ y'
+    expr = r'~ x /\ y'
     _test_reduction_complemented_edges(expr, bdd)
     # complemented edge from external reference to
     # non-terminal node
-    expr = 'x /\ ~ y'
+    expr = r'x /\ ~ y'
     u = bdd.add_expr(expr)
     assert u < 0, u
     _test_reduction_complemented_edges(expr, bdd)
@@ -464,10 +472,10 @@ def test_find_or_add():
     assert refv == refv_, (refv, refv_)
     assert refw == refw_, (refw, refw_)
     # only non-terminals can be added
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.find_or_add(2, -1, 1)
     # low and high must already exist
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.find_or_add(0, 3, 4)
     # canonicity of complemented edges
     # v < 0, w > 0
@@ -519,7 +527,7 @@ def test_next_free_int():
     # full
     g._succ = {1, 2, 3}
     g.max_nodes = 3
-    with nt.assert_raises(Exception):
+    with pytest.raises(Exception):
         g._next_free_int(start=1)
 
 
@@ -531,7 +539,7 @@ def _assert_smaller_are_nodes(start, bdd):
 def test_collect_garbage():
     # all nodes are garbage
     g = BDD({'x': 0, 'y': 1})
-    u = g.add_expr('x /\ y')
+    u = g.add_expr(r'x /\ y')
     n = len(g)
     assert n == 4, n
     uref = g._ref[abs(u)]
@@ -550,7 +558,7 @@ def test_collect_garbage():
     # some nodes not garbage
     # projection of x is garbage
     g = BDD({'x': 0, 'y': 1})
-    u = g.add_expr('x /\ y')
+    u = g.add_expr(r'x /\ y')
     n = len(g)
     assert n == 4, n
     g._ref[abs(u)] += 1
@@ -609,7 +617,7 @@ def test_add_expr():
     h = ref_var(ix)
     compare(u, g, h)
     # x and y
-    u = g.add_expr('x /\ y')
+    u = g.add_expr(r'x /\ y')
     h = ref_x_and_y()
     compare(u, g, h)
 
@@ -618,27 +626,27 @@ def test_compose():
     ordering = {'x': 0, 'y': 1, 'z': 2}
     g = BDD(ordering)
     # x /\ (x \/ z)
-    a = g.add_expr('x /\ y')
-    b = g.add_expr('x \/ z')
+    a = g.add_expr(r'x /\ y')
+    b = g.add_expr(r'x \/ z')
     c = g.let({'y': b}, a)
-    d = g.add_expr('x /\ (x \/ z)')
+    d = g.add_expr(r'x /\ (x \/ z)')
     assert c == d, (c, d)
     # (y \/ z) /\ x
     ordering = {'x': 0, 'y': 1, 'z': 2, 'w': 3}
     g = BDD(ordering)
-    a = g.add_expr('(x /\ y) \/ z')
-    b = g.add_expr('(y \/ z) /\ x')
+    a = g.add_expr(r'(x /\ y) \/ z')
+    b = g.add_expr(r'(y \/ z) /\ x')
     c = g.let({'z': b}, a)
     assert c == b, (c, b)
     # long expr
     ordering = {'x': 0, 'y': 1, 'z': 2, 'w': 3}
     g = BDD(ordering)
-    a = g.add_expr('(x /\ y) \/ (~ z \/ (w /\ y /\ x))')
-    b = g.add_expr('(y \/ ~ z) /\ x')
+    a = g.add_expr(r'(x /\ y) \/ (~ z \/ (w /\ y /\ x))')
+    b = g.add_expr(r'(y \/ ~ z) /\ x')
     c = g.let({'y': b}, a)
     d = g.add_expr(
-        '(x /\ ((y \/ ~ z) /\ x)) \/ '
-        ' (~ z \/ (w /\ ((y \/ ~ z) /\ x) /\ x))')
+        r'(x /\ ((y \/ ~ z) /\ x)) \/ '
+        r' (~ z \/ (w /\ ((y \/ ~ z) /\ x) /\ x))')
     assert c == d, (c, d)
     # complemented edges
     ordering = {'x': 0, 'y': 1}
@@ -655,10 +663,10 @@ def test_cofactor():
     ordering = {'x': 0, 'y': 1, 'z': 2}
     g = BDD(ordering)
     # u not in g
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.let({'x': False, 'y': True, 'z': False}, 5)
     # x /\ y
-    e = g.add_expr('x /\ y')
+    e = g.add_expr(r'x /\ y')
     x = g.add_expr('x')
     assert g.let({'x': False}, x) == -1
     assert g.let({'x': True}, x) == 1
@@ -700,19 +708,19 @@ def test_swap():
     assert g._ref[abs(y)] == 1
     # x /\ y
     g = BDD({'x': 0, 'y': 1})
-    u = g.add_expr('x /\ y')
+    u = g.add_expr(r'x /\ y')
     g.incref(u)
     nold, n = g.swap('x', 'y')
     assert nold == n, (nold, n)
     assert g.vars == {'y': 0, 'x': 1}, g.vars
-    u_ = g.add_expr('x /\ y')
+    u_ = g.add_expr(r'x /\ y')
     assert u == u_, (u, u_)
     assert g.assert_consistent()
     # reference counts unchanged
     assert g._ref[abs(u)] == 1
     # x /\ ~ y
     # tests handling of complement edges
-    e = 'x /\ ~ y'
+    e = r'x /\ ~ y'
     g = x_and_not_y()
     u = g.add_expr(e)
     g.incref(u)
@@ -739,7 +747,7 @@ def test_swap():
     # Figs. 6.24, 6.25 Baier 2008
     g = BDD({'z1': 0, 'y1': 1, 'z2': 2,
              'y2': 3, 'z3': 4, 'y3': 5})
-    u = g.add_expr('(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
+    u = g.add_expr(r'(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
     g.incref(u)
     n = len(g)
     assert n == 16, n
@@ -757,7 +765,7 @@ def test_swap():
         'z1': 0, 'z2': 1, 'z3': 2,
         'y1': 3, 'y2': 4, 'y3': 5}
     assert g.vars == new_ordering, g.vars
-    u_ = g.add_expr('(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
+    u_ = g.add_expr(r'(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
     assert u_ == u, (u, u_, g._succ)
     # g.dump('g.pdf')
 
@@ -766,7 +774,7 @@ def test_sifting():
     # Figs. 6.24, 6.25 Baier 2008
     g = BDD({'z1': 0, 'z2': 1, 'z3': 2,
              'y1': 3, 'y2': 4, 'y3': 5})
-    u = g.add_expr('(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
+    u = g.add_expr(r'(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
     g.incref(u)
     g.collect_garbage()
     n = len(g)
@@ -774,7 +782,7 @@ def test_sifting():
     _bdd.reorder(g)
     n_ = len(g)
     assert n > n_, (n, n_)
-    u_ = g.add_expr('(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
+    u_ = g.add_expr(r'(z1 /\ y1) \/ (z2 /\ y2) \/ (z3 /\ y3)')
     g.incref(u)
     g.collect_garbage()
     g.assert_consistent()
@@ -791,7 +799,7 @@ def test_request_reordering():
     ctx._last_len = 1
     ctx.length = 3  # >= 2 = 2 * _last_len
     # large growth
-    with nt.assert_raises(_bdd._NeedsReordering):
+    with pytest.raises(_bdd._NeedsReordering):
         _bdd._request_reordering(ctx)
     ctx._last_len = 2
     ctx.length = 3  # < 4 = 2 * _last_len
@@ -809,20 +817,20 @@ def test_reordering_context():
     ctx.assert_(False)
     # nested context
     ctx._reordering_context = True
-    with nt.assert_raises(_bdd._NeedsReordering):
+    with pytest.raises(_bdd._NeedsReordering):
         with _bdd._ReorderingContext(ctx):
             ctx.assert_(True)
             raise _bdd._NeedsReordering()
     ctx.assert_(True)
     # other exception
     ctx._reordering_context = False
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         with _bdd._ReorderingContext(ctx):
             ctx.assert_(True)
             raise AssertionError()
     ctx.assert_(False)
     ctx._reordering_context = True
-    with nt.assert_raises(Exception):
+    with pytest.raises(Exception):
         with _bdd._ReorderingContext(ctx):
             raise Exception()
     ctx.assert_(True)
@@ -850,7 +858,7 @@ def test_dynamic_reordering():
     # add expr with reordering off
     assert not b.reordering_is_on()
     assert b.n_swaps == 0, b.n_swaps
-    u = b.add_expr('x /\ y /\ z')
+    u = b.add_expr(r'x /\ y /\ z')
     assert b.n_swaps == 0, b.n_swaps
     b.incref(u)
     n = len(b)
@@ -858,7 +866,7 @@ def test_dynamic_reordering():
     # add expr with reordering on
     b._last_len = 6
     assert b.reordering_is_on()
-    v = b.add_expr('a /\ b')
+    v = b.add_expr(r'a /\ b')
     assert b.reordering_is_on()
     assert b.n_swaps == 0, b.n_swaps
     b.incref(v)
@@ -866,7 +874,7 @@ def test_dynamic_reordering():
     assert n == 10, n
     # add an expr that triggers reordering
     assert b.reordering_is_on()
-    w = b.add_expr('z \/ (~ a /\ x /\ ~ y)')
+    w = b.add_expr(r'z \/ (~ a /\ x /\ ~ y)')
     assert b.reordering_is_on()
     n_swaps = b.n_swaps
     assert n_swaps > 0, n_swaps
@@ -877,8 +885,8 @@ def test_dynamic_reordering():
     # add another expr that triggers reordering
     old_n_swaps = n_swaps
     assert b.reordering_is_on()
-    r = b.add_expr('(~ z \/ (c /\ b)) /\ e /\ (a /\ (~x \/ y))')
-    b.add_expr('(e \/ ~ a) /\ x /\ (b \/ ~ y)')
+    r = b.add_expr(r'(~ z \/ (c /\ b)) /\ e /\ (a /\ (~x \/ y))')
+    b.add_expr(r'(e \/ ~ a) /\ x /\ (b \/ ~ y)')
     n_swaps = b.n_swaps
     assert n_swaps > old_n_swaps, (n_swaps, old_n_swaps)
     assert b.reordering_is_on()
@@ -905,7 +913,7 @@ def test_undeclare_vars():
     bdd = BDD()
     bdd.declare('x', 'y', 'z', 'w')
     # empty arg `vrs`
-    u = bdd.add_expr('x /\ y /\ w')
+    u = bdd.add_expr(r'x /\ y /\ w')
     rm_vars = bdd.undeclare_vars()
     rm_vars_ = {'z'}
     assert rm_vars == rm_vars_, (rm_vars, rm_vars_)
@@ -915,7 +923,7 @@ def test_undeclare_vars():
     # nonempty `vrs` with all empty levels
     bdd = BDD()
     bdd.declare('x', 'y', 'z', 'w')
-    u = bdd.add_expr('y /\ w')
+    u = bdd.add_expr(r'y /\ w')
     rm_vars = bdd.undeclare_vars('x', 'z')
     rm_vars_ = {'x', 'z'}
     assert rm_vars == rm_vars_, (rm_vars, rm_vars_)
@@ -925,7 +933,7 @@ def test_undeclare_vars():
     # nonempty `vrs` without all empty levels
     bdd = BDD()
     bdd.declare('x', 'y', 'z', 'w')
-    u = bdd.add_expr('y /\ w')
+    u = bdd.add_expr(r'y /\ w')
     rm_vars = bdd.undeclare_vars('z')
     rm_vars_ = {'z'}
     assert rm_vars == rm_vars_, (rm_vars, rm_vars_)
@@ -935,8 +943,8 @@ def test_undeclare_vars():
     # remove only unused variables
     bdd = BDD()
     bdd.declare('x', 'y', 'z', 'w')
-    u = bdd.add_expr('y /\ w')
-    with nt.assert_raises(AssertionError):
+    u = bdd.add_expr(r'y /\ w')
+    with pytest.raises(AssertionError):
         bdd.undeclare_vars('z', 'y')
 
 
@@ -946,13 +954,13 @@ def test_dump_load():
     dvars = dict(x=0, y=1)
     # dump
     b = BDD(dvars)
-    e = 'x /\ ~ y'
+    e = r'x /\ ~ y'
     u_dumped = b.add_expr(e)
     b.dump(fname, [u_dumped])
     b.dump(fname)  # no roots
     # load
     b = BDD(dvars)
-    b.add_expr('x \/ y')
+    b.add_expr(r'x \/ y')
     u_new = b.add_expr(e)
     umap = b.load(fname)
     u_loaded = umap[abs(u_dumped)]
@@ -966,7 +974,7 @@ def test_dump_load():
 def test_dump_load_manager():
     prefix = 'test_dump_load_manager'
     g = BDD({'x': 0, 'y': 1})
-    e = 'x /\ ~ y'
+    e = r'x /\ ~ y'
     u = g.add_expr(e)
     g.incref(u)
     fname = prefix + '.p'
@@ -978,13 +986,24 @@ def test_dump_load_manager():
     # h.dump(prefix + '.pdf')
 
 
-def test_dump_pdf():
+def test_dump_using_graphviz():
     bdd = BDD()
     bdd.declare('x', 'y', 'z')
     u = bdd.add_expr(r'x /\ y')
     v = bdd.add_expr(r'y /\ ~ z')
-    fname = 'bdd.pdf'
     roots = [u, v]
+    filename_noext = 'bdd'
+    filetypes = ['pdf', 'png', 'svg']
+    for filetype in filetypes:
+        _dump_bdd_roots_as_filetype(
+            roots, bdd, filename_noext, filetype)
+
+
+def _dump_bdd_roots_as_filetype(
+        roots, bdd, filename_noext, filetype):
+    fname = '{name}.{ext}'.format(
+        name=filename_noext,
+        ext=filetype)
     if os.path.isfile(fname):
         os.remove(fname)
     bdd.dump(fname, roots)
@@ -995,7 +1014,7 @@ def test_quantify():
     ordering = {'x': 0, 'y': 1, 'z': 2}
     g = BDD(ordering)
     # x /\ y
-    e = g.add_expr('x /\ ~ y')
+    e = g.add_expr(r'x /\ ~ y')
     x = g.add_expr('x')
     not_y = g.add_expr('~ y')
     assert g.quantify(e, {'x'}) == not_y
@@ -1003,10 +1022,10 @@ def test_quantify():
     assert g.quantify(e, {'y'}) == x
     assert g.quantify(e, {'x'}, forall=True) == -1
     # x \/ y \/ z
-    e = g.add_expr('x \/ y \/ z')
-    xy = g.add_expr('x \/ y')
-    yz = g.add_expr('y \/ z')
-    zx = g.add_expr('z \/ x')
+    e = g.add_expr(r'x \/ y \/ z')
+    xy = g.add_expr(r'x \/ y')
+    yz = g.add_expr(r'y \/ z')
+    zx = g.add_expr(r'z \/ x')
     assert g.quantify(e, {'x'})
     assert g.quantify(e, {'y'})
     assert g.quantify(e, {'z'})
@@ -1018,7 +1037,7 @@ def test_quantify():
     v = g.quantify(u, {'y'}, forall=True)
     assert v == -x, g.to_expr(v)
     # multiple values: test recursion
-    e = g.add_expr('x /\ y /\ z')
+    e = g.add_expr(r'x /\ y /\ z')
     x = g.add_expr('x')
     r = g.quantify(e, {'y', 'z'})
     assert r == x, r
@@ -1028,30 +1047,30 @@ def test_quantifier_syntax():
     b = BDD()
     [b.add_var(var) for var in ['x', 'y']]
     # constants
-    u = b.add_expr('\E x:  TRUE')
+    u = b.add_expr(r'\E x:  TRUE')
     assert u == b.true, u
-    u = b.add_expr('\E x, y:  TRUE')
+    u = b.add_expr(r'\E x, y:  TRUE')
     assert u == b.true, u
-    u = b.add_expr('\E x:  FALSE')
+    u = b.add_expr(r'\E x:  FALSE')
     assert u == b.false, u
-    u = b.add_expr('\A x:  TRUE')
+    u = b.add_expr(r'\A x:  TRUE')
     assert u == b.true, u
-    u = b.add_expr('\A x:  FALSE')
+    u = b.add_expr(r'\A x:  FALSE')
     assert u == b.false, u
-    u = b.add_expr('\A x, y:  FALSE')
+    u = b.add_expr(r'\A x, y:  FALSE')
     assert u == b.false, u
     # variables
-    u = b.add_expr('\E x:  x')
+    u = b.add_expr(r'\E x:  x')
     assert u == b.true, u
-    u = b.add_expr('\A x:  x')
+    u = b.add_expr(r'\A x:  x')
     assert u == b.false, u
-    u = b.add_expr('\E x, y:  x')
+    u = b.add_expr(r'\E x, y:  x')
     assert u == b.true, u
-    u = b.add_expr('\E x, y:  y')
+    u = b.add_expr(r'\E x, y:  y')
     assert u == b.true, u
-    u = b.add_expr('\A x:  y')
+    u = b.add_expr(r'\A x:  y')
     assert u == b.var('y'), u
-    u = b.add_expr('\A x:  ~ y')
+    u = b.add_expr(r'\A x:  ~ y')
     u_ = b.apply('not', b.var('y'))
     assert u == u_, (u, u_)
 
@@ -1068,25 +1087,25 @@ def test_rename():
                 'y': 2, 'yp': 3,
                 'z': 4, 'zp': 5}
     g = BDD(ordering)
-    u = g.add_expr('x /\ y /\ ~ z')
+    u = g.add_expr(r'x /\ y /\ ~ z')
     dvars = {'x': 'xp', 'y': 'yp', 'z': 'zp'}
     urenamed = g.let(dvars, u)
-    up = g.add_expr('xp /\ yp /\ ~ zp')
+    up = g.add_expr(r'xp /\ yp /\ ~ zp')
     assert urenamed == up, urenamed
     # assertion violations
     # non-neighbors
     dvars = {'x': 'yp'}
     r = g.let(dvars, u)
-    r_ = g.add_expr('yp /\ y /\ ~ z')
+    r_ = g.add_expr(r'yp /\ y /\ ~ z')
     assert r == r_, (r, r_)
     # u not in bdd
     dvars = {'x': 'xp'}
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         g.let(dvars, 1000)
     # y essential for u
     dvars = {'x': 'y'}
     v = g.let(dvars, u)
-    v_ = g.add_expr('y /\ ~ z')
+    v_ = g.add_expr(r'y /\ ~ z')
     assert v == v_, (v, v_)
     # old and new vars intersect
     dvars = {'x': 'x'}
@@ -1098,25 +1117,25 @@ def test_rename_syntax():
     b = BDD()
     [b.add_var(var) for var in ['x', 'y', 'z', 'w']]
     # single substitution
-    u = b.add_expr('\S y / x:  TRUE')
+    u = b.add_expr(r'\S y / x:  TRUE')
     assert u == b.true, u
-    u = b.add_expr('\S y / x:  FALSE')
+    u = b.add_expr(r'\S y / x:  FALSE')
     assert u == b.false, u
-    u = b.add_expr('\S y / x:  x')
+    u = b.add_expr(r'\S y / x:  x')
     u_ = b.add_expr('y')
     assert u == u_, (u, u_)
-    u = b.add_expr('\S y / x:  z')
+    u = b.add_expr(r'\S y / x:  z')
     u_ = b.add_expr('z')
     assert u == u_, (u, u_)
-    u = b.add_expr('\S y / x:  x /\ z')
-    u_ = b.add_expr('y /\ z')
+    u = b.add_expr(r'\S y / x:  x /\ z')
+    u_ = b.add_expr(r'y /\ z')
     assert u == u_, (u, u_)
     # multiple substitution
-    u = b.add_expr('\S y / x,  w / z:  x /\ z')
-    u_ = b.add_expr('y /\ w')
+    u = b.add_expr(r'\S y / x,  w / z:  x /\ z')
+    u_ = b.add_expr(r'y /\ w')
     assert u == u_, (u, u_)
-    u = b.add_expr('\S y / x,  w / z:  z \/ ~ x')
-    u_ = b.add_expr('w \/ ~ y')
+    u = b.add_expr(r'\S y / x,  w / z:  z \/ ~ x')
+    u_ = b.add_expr(r'w \/ ~ y')
     assert u == u_, (u, u_)
 
 
@@ -1134,21 +1153,21 @@ def test_image_rename_map_checks():
     assert r == 1, r
     # overlapping keys and values
     rename = {0: 1, 1: 2}
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         _bdd.image(1, 1, rename, qvars, bdd)
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         _bdd.preimage(1, 1, rename, qvars, bdd)
     # may be in support after quantification ?
     trans = bdd.add_expr('x => xp')
-    source = bdd.add_expr('x /\ y')
+    source = bdd.add_expr(r'x /\ y')
     qvars = {0}
     rename = {1: 0, 3: 2}
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         _bdd.image(trans, source, rename, qvars, bdd)
     # in support of `target` ?
     qvars = set()
     trans = bdd.add_expr('y')
-    target = bdd.add_expr('x /\ y')
+    target = bdd.add_expr(r'x /\ y')
     rename = {0: 2}
     r = _bdd.preimage(trans, target, rename, qvars, bdd)
     assert r == bdd.var('y'), r
@@ -1172,18 +1191,18 @@ def test_preimage():
     # (x /\ y) --> (~ x /\ y) -->
     # (~ x /\ ~ y) --> (x /\ ~ y) --> wrap around
     t = g.add_expr(
-        '((x /\ y) => (~ xp /\ yp)) /\ '
-        '((~ x /\ y) => (~ xp /\ ~ yp)) /\ '
-        '((~ x /\ ~ y) => (xp /\ ~ yp)) /\ '
-        '((x /\ ~ y) => (xp /\ yp))')
-    f = g.add_expr('x /\ y')
+        r'((x /\ y) => (~ xp /\ yp)) /\ '
+        r'((~ x /\ y) => (~ xp /\ ~ yp)) /\ '
+        r'((~ x /\ ~ y) => (xp /\ ~ yp)) /\ '
+        r'((x /\ ~ y) => (xp /\ yp))')
+    f = g.add_expr(r'x /\ y')
     p = preimage(t, f, rename, qvars, g)
-    assert p == g.add_expr('x /\ ~ y')
-    f = g.add_expr('x /\ ~ y')
+    assert p == g.add_expr(r'x /\ ~ y')
+    f = g.add_expr(r'x /\ ~ y')
     p = preimage(t, f, rename, qvars, g)
-    assert p == g.add_expr('~ x /\ ~ y')
+    assert p == g.add_expr(r'~ x /\ ~ y')
     # backward reachable set
-    f = g.add_expr('x /\ y')
+    f = g.add_expr(r'x /\ y')
     oldf = None
     while oldf != f:
         p = preimage(t, f, rename, qvars, g)
@@ -1191,7 +1210,7 @@ def test_preimage():
         f = g.apply('or', p, oldf)
     assert f == 1
     # go around once
-    f = g.add_expr('x /\ y')
+    f = g.add_expr(r'x /\ y')
     start = f
     for i in range(4):
         f = preimage(t, f, rename, qvars, g)
@@ -1199,26 +1218,26 @@ def test_preimage():
     assert start == end
     # forall z exists x, y
     t = g.add_expr(
-        '('
-        '    ((x /\ y) => (zp /\ xp /\ ~ yp)) \/ '
-        '    ((x /\ y) => (~ zp /\ ~ xp /\ yp))'
-        ') /\ '
-        '(~ (x /\ y) => False)')
-    f = g.add_expr('x /\ ~ y')
+        r'('
+        r'    ((x /\ y) => (zp /\ xp /\ ~ yp)) \/ '
+        r'    ((x /\ y) => (~ zp /\ ~ xp /\ yp))'
+        r') /\ '
+        r'(~ (x /\ y) => False)')
+    f = g.add_expr(r'x /\ ~ y')
     ep = preimage(t, f, rename, qvars, g)
     p = g.quantify(ep, {'zp'}, forall=True)
     assert p == -1
-    f = g.add_expr('(x /\ ~ y) \/ (~ x /\ y)')
+    f = g.add_expr(r'(x /\ ~ y) \/ (~ x /\ y)')
     ep = preimage(t, f, rename, qvars, g)
     p = g.quantify(ep, {'zp'}, forall=True)
-    assert p == g.add_expr('x /\ y')
+    assert p == g.add_expr(r'x /\ y')
 
 
 def test_assert_valid_ordering():
     ordering = {'x': 0, 'y': 1}
     _bdd._assert_valid_ordering(ordering)
     incorrect_ordering = {'x': 0, 'y': 2}
-    with nt.assert_raises(AssertionError):
+    with pytest.raises(AssertionError):
         _bdd._assert_valid_ordering(incorrect_ordering)
 
 
@@ -1228,33 +1247,58 @@ def test_assert_refined_ordering():
     _bdd._assert_isomorphic_orders(ordering, new_ordering, ordering)
 
 
-def test_to_pydot():
-    def f(x):
+def test_to_graphviz_dot():
+    def fmt(x):
         return str(abs(x))
     # with roots
     g = x_and_y()
     pd = _bdd.to_pydot([4, 2], g)
-    r = nx.drawing.nx_pydot.from_pydot(pd)
+    r = _graph_from_dot(pd)
     for u in g:
-        assert f(u) in r, (u, r.nodes())
-    for u in g._succ:
-        i, v, w = g._succ[u]
+        assert fmt(u) in r, (u, r)
+    for u, (_, v, w) in g._succ.items():
+        su = fmt(u)
+        assert su in r, (su, r)
         if v is None or w is None:
             assert v is None, v
             assert w is None, w
             continue
-        assert r.has_edge(f(u), f(v)), (u, v)
-        assert r.has_edge(f(u), f(w)), (u, w)
+        sv = fmt(v)
+        sw = fmt(w)
+        assert sv in r[su], (su, sv, r)
+        assert sw in r[su], (su, sw, r)
     # no roots
     pd = _bdd.to_pydot(None, g)
-    r = nx.drawing.nx_pydot.from_pydot(pd)
-    assert len(r) == 8, r.nodes()  # 3 hidden nodes for levels
+    r = _graph_from_dot(pd)
+    # `r` has 3 hidden nodes,
+    # used to layout variable levels
+    assert len(r) == 8, r
+
+
+def _graph_from_dot(dot_graph):
+    """Return `dict` of `set` for graph."""
+    g = dict()
+    return _graph_from_dot_recurse(dot_graph, g)
+
+
+def _graph_from_dot_recurse(dot_graph, g):
+    for h in dot_graph.get_subgraphs():
+        _graph_from_dot_recurse(h, g)
+    for node in dot_graph.get_nodes():
+        name = node.get_name()
+        g[name] = set()
+    for edge in dot_graph.get_edges():
+        src = edge.get_source()
+        dst = edge.get_destination()
+        assert src in g, (src, g)
+        g[src].add(dst)
+    return g
 
 
 def test_function_wrapper():
     levels = dict(x=0, y=1, z=2)
     bdd = autoref.BDD(levels)
-    u = bdd.add_expr('x /\ y')
+    u = bdd.add_expr(r'x /\ y')
     assert u.bdd is bdd, (repr(u.bdd), repr(bdd))
     assert abs(u.node) in bdd._bdd, (u.node, bdd._bdd._succ)
     # operators
@@ -1262,12 +1306,12 @@ def test_function_wrapper():
     z = bdd.add_expr('z')
     v = x.implies(z)
     w = u & ~ v
-    w_ = bdd.add_expr('(x /\ y) /\ ~ ((~ x) \/ z)')
+    w_ = bdd.add_expr(r'(x /\ y) /\ ~ ((~ x) \/ z)')
     assert w_ == w, (w_, w)
     r = ~ (u | v).equiv(w)
     r_ = bdd.add_expr(
-        '( (x /\ y) \/ ((~ x) \/ z) ) ^'
-        '( (x /\ y) /\ ~ ((~ x) \/ z) )')
+        r'( (x /\ y) \/ ((~ x) \/ z) ) ^'
+        r'( (x /\ y) /\ ~ ((~ x) \/ z) )')
     assert r_ == r, (r_, r)
     p = bdd.add_expr('y')
     q = p.equiv(x)
@@ -1304,7 +1348,7 @@ def test_function_wrapper():
     assert n == 1, bdd._bdd._ref
     # properties
     bdd = autoref.BDD({'x': 0, 'y': 1, 'z': 2})
-    u = bdd.add_expr('x \/ ~ y')
+    u = bdd.add_expr(r'x \/ ~ y')
     assert u.level == 0, u.level
     assert u.var == 'x', u.var
     y = bdd.add_expr('~ y')
